@@ -5,6 +5,8 @@
 
 package beagle;
 
+import java.io.Serializable;
+
 /**
  * Beagle - An interface exposing the BEAGLE likelihood evaluation library.
  *
@@ -18,7 +20,7 @@ package beagle;
  * @version $Id:$
  */
 
-public interface Beagle {
+public interface Beagle extends Serializable {
 
     public static int OPERATION_TUPLE_SIZE = 7;
     public static int NONE = -1;
@@ -109,7 +111,20 @@ public interface Beagle {
     void getPartials(
             int bufferIndex,
             int scaleIndex,
-            final double []outPartials);
+            final double[] outPartials);
+                        
+    /**
+     * Get scale factors from instance buffer on log-scale
+     *
+     * This function copies an array of scale factors from an instance buffer. The outFactors array should
+     * be patternCount in length.
+     *   
+     * @param scaleIndex    Index of scaleBuffer to get (input)
+     * @param  outFactors  Pointer to which to receive partialsBuffer (output)
+     */
+    void getLogScaleFactors(           
+            int scaleIndex,
+            final double[] outFactors);            
 
     /**
      * Set an eigen-decomposition buffer
