@@ -61,7 +61,8 @@ public class BeagleJNIWrapper {
 
     public native int getPartials(int instance, int bufferIndex, int scaleIndex,
                                   final double[] outPartials);
-
+    
+    public native int getLogScaleFactors(int stance, int scaleIndex, final double[] outFactors);
 
     public native int setEigenDecomposition(int instance,
                                             int eigenIndex,
@@ -155,7 +156,7 @@ public class BeagleJNIWrapper {
         String osName = System.getProperty("os.name").toLowerCase();
         String osArch = System.getProperty("os.arch").toLowerCase();
         if (osName.startsWith("windows")) {
-            if(osArch.equals("i386")) return "hmsbeagle32";
+            if(osArch.equals("x86")||osArch.equals("i386")) return "hmsbeagle32";
             if(osArch.startsWith("amd64")||osArch.startsWith("x86_64")) return "hmsbeagle64";
         }
         return "hmsbeagle-jni";
