@@ -182,6 +182,15 @@ public class GeneralBeagleImpl implements Beagle {
         System.arraycopy(this.partials[bufferIndex], 0, partials, 0, partialsSize);
     }
 
+    @Override
+    public void getLogScaleFactors(int scaleIndex, double[] outFactors) {
+        throw new UnsupportedOperationException("Not implemented. Email Marc Suchard if required (offer coauthorship for enhanced service)");
+    }
+
+    public void getScaleFactors(final int scaleIndex, final double[] partials) {
+        throw new RuntimeException("Not yet implemented.");
+    }
+
     public void setEigenDecomposition(int eigenIndex, double[] eigenVectors, double[] inverseEigenValues, double[] eigenValues) {
         int l =0;
         for (int i = 0; i < stateCount; i++) {
