@@ -1232,6 +1232,13 @@ int BeagleCPUImpl<BEAGLE_CPU_GENERIC>::copyScaleFactors(int destScalingIndex,
     return BEAGLE_SUCCESS;
 }
 
+BEAGLE_CPU_TEMPLATE
+int BeagleCPUImpl<BEAGLE_CPU_GENERIC>::getScaleFactors(int srcScalingIndex,
+                        							   double* scaleFactors) {
+	// Do nothing                        							   
+	return BEAGLE_SUCCESS;                        							   
+}                        							   
+
 BEAGLE_CPU_TEMPLATE
     int BeagleCPUImpl<BEAGLE_CPU_GENERIC>::calculateEdgeLogLikelihoods(const int* parentBufferIndices,
                                                              const int* childBufferIndices,
