@@ -29,6 +29,7 @@
 
 #include <cstdio>
 #include <cstdlib>
+#include <cstring>
 
 #include "libhmsbeagle/beagle.h"
 #include "libhmsbeagle/GPU/GPUImplDefs.h"
@@ -36,6 +37,12 @@
 
 /**************CODE***********/
 
+#ifdef CUDA
+namespace cuda_device {
+#else
+namespace opencl_device {
+#endif
+
 REAL* ones = NULL; // TODO: Memory leak, need to free at some point.
 
 KernelLauncher::KernelLauncher(GPUInterface* inGpu) {
@@ -48,10 +55,25 @@ KernelLauncher::~KernelLauncher() {
 }
 
 void KernelLauncher::SetupKernelBlocksAndGrids() {
+    bool CPUImplementation = false;
+    bool AppleCPUImplementation = false;
+
+#ifdef FW_OPENCL
+    BeagleDeviceImplementationCodes deviceCode = gpu->GetDeviceImplementationCode(-1);
+    if (deviceCode == BEAGLE_OPENCL_DEVICE_INTEL_CPU ||
+        deviceCode == BEAGLE_OPENCL_DEVICE_INTEL_MIC ||
+        deviceCode == BEAGLE_OPENCL_DEVICE_AMD_CPU) {
+        CPUImplementation = true;
+    } else if (deviceCode == BEAGLE_OPENCL_DEVICE_APPLE_CPU) {
+        CPUImplementation = true;
+        AppleCPUImplementation = true;
+    }
+#endif
 
     kPaddedStateCount = gpu->kernelResource->paddedStateCount;
     kCategoryCount = gpu->kernelResource->categoryCount;
     kPatternCount = gpu->kernelResource->patternCount;
+    kUnpaddedPatternCount = gpu->kernelResource->unpaddedPatternCount;
     kMultiplyBlockSize = gpu->kernelResource->multiplyBlockSize;
     kPatternBlockSize = gpu->kernelResource->patternBlockSize;
     kSlowReweighing = gpu->kernelResource->slowReweighing;
@@ -61,36 +83,59 @@ void KernelLauncher::SetupKernelBlocksAndGrids() {
     
     // Set up block/grid for transition matrices computation
     bgTransitionProbabilitiesBlock = Dim3Int(kMultiplyBlockSize, kMultiplyBlockSize);
-    bgTransitionProbabilitiesGrid = Dim3Int(
-            kPaddedStateCount/kMultiplyBlockSize, 
-            kPaddedStateCount/kMultiplyBlockSize);
+    bgTransitionProbabilitiesGrid = Dim3Int(kPaddedStateCount/kMultiplyBlockSize, 
+                                            kPaddedStateCount/kMultiplyBlockSize);
     if(kPaddedStateCount % kMultiplyBlockSize != 0) {
         bgTransitionProbabilitiesGrid.x += 1;
         bgTransitionProbabilitiesGrid.y += 1;
     }
-    
+
     // Set up block/grid for peeling computation
     if (kPaddedStateCount == 4) {
-        bgPeelingBlock = Dim3Int(16, kPatternBlockSize);
-        bgPeelingGrid  = Dim3Int(kPatternCount / (kPatternBlockSize * 4), kCategoryCount);
-        if (kPatternCount % (kPatternBlockSize * 4) != 0)
-            bgPeelingGrid.x += 1;
+        if (CPUImplementation) {
+            bgPeelingBlock = Dim3Int(kPatternBlockSize, 1);
+            bgPeelingGrid  = Dim3Int(kPatternCount / kPatternBlockSize, kCategoryCount);
+        } else {
+            bgPeelingBlock = Dim3Int(16, kPatternBlockSize);
+            bgPeelingGrid  = Dim3Int(kPatternCount / (kPatternBlockSize * 4),
+                                     kCategoryCount);
+            if (kPatternCount % (kPatternBlockSize * 4) != 0) {
+                bgPeelingGrid.x += 1;
+            }
+        }
     } else {
-        bgPeelingBlock = Dim3Int(kPaddedStateCount, kPatternBlockSize);
-        bgPeelingGrid  = Dim3Int(kPatternCount / kPatternBlockSize, kCategoryCount);
-        if (kPatternCount % kPatternBlockSize != 0)
+        if (AppleCPUImplementation) {
+            bgPeelingBlock = Dim3Int(kPaddedStateCount, 1, 1);
+            bgPeelingGrid  = Dim3Int(kPatternCount / kPatternBlockSize, kPatternBlockSize, kCategoryCount);
+        } else if (CPUImplementation) {
+            bgPeelingBlock = Dim3Int(kPaddedStateCount, kPatternBlockSize, 1);
+            bgPeelingGrid  = Dim3Int(kPatternCount / kPatternBlockSize, 1, kCategoryCount);
+        } else {
+            bgPeelingBlock = Dim3Int(kPaddedStateCount, kPatternBlockSize);
+            bgPeelingGrid  = Dim3Int(kPatternCount / kPatternBlockSize, kCategoryCount);
+        }
+        if (!CPUImplementation && (kPatternCount % kPatternBlockSize != 0)) {
             bgPeelingGrid.x += 1;
+        }
     } 
-    
+
     // Set up block/grid for likelihood computation
     if (kPaddedStateCount == 4) {
         int likePatternBlockSize = kPatternBlockSize;
-        bgLikelihoodBlock = Dim3Int(4,likePatternBlockSize);
+        if (CPUImplementation) {
+            bgLikelihoodBlock = Dim3Int(likePatternBlockSize,1);
+        } else {
+            bgLikelihoodBlock = Dim3Int(4,likePatternBlockSize);
+        }
         bgLikelihoodGrid = Dim3Int(kPatternCount/likePatternBlockSize);
         if (kPatternCount % likePatternBlockSize != 0)
             bgLikelihoodGrid.x += 1;
     } else {
-        bgLikelihoodBlock = Dim3Int(kPaddedStateCount);
+        if (CPUImplementation) {
+            bgLikelihoodBlock = Dim3Int(1);
+        } else {
+            bgLikelihoodBlock = Dim3Int(kPaddedStateCount);
+        }
         bgLikelihoodGrid  = Dim3Int(kPatternCount);
     }
     
@@ -105,46 +150,56 @@ void KernelLauncher::SetupKernelBlocksAndGrids() {
         bgAccumulateGrid.x += 1;        
     
     // Set up block/grid for scaling partials
-    if (kSlowReweighing) {
-        bgScaleBlock = Dim3Int(kPaddedStateCount);
-        bgScaleGrid  = Dim3Int(kPatternCount);        
+    if (CPUImplementation) {
+        bgScaleBlock = Dim3Int(kPatternBlockSize);
+        bgScaleGrid  = Dim3Int(kPatternCount/kPatternBlockSize);
     } else {
-        if (kPaddedStateCount == 4) {
-            bgScaleBlock = Dim3Int(16, kMatrixBlockSize);
-            bgScaleGrid  = Dim3Int(kPatternCount / 4, kCategoryCount/kMatrixBlockSize);
-            if (kPatternCount % 4 != 0) {
-                bgScaleGrid.x += 1; // 
-                fprintf(stderr,"PATTERNS SHOULD BE PADDED! Inform Marc, please.\n");
-                exit(-1);
+        if (kSlowReweighing) {
+            bgScaleBlock = Dim3Int(kPaddedStateCount);
+            bgScaleGrid  = Dim3Int(kPatternCount);        
+        } else {
+            if (kPaddedStateCount == 4) {
+                bgScaleBlock = Dim3Int(16, kMatrixBlockSize);
+                bgScaleGrid  = Dim3Int(kPatternCount / 4, kCategoryCount/kMatrixBlockSize);
+                if (kPatternCount % 4 != 0) {
+                    bgScaleGrid.x += 1; // 
+                    fprintf(stderr,"PATTERNS SHOULD BE PADDED! Inform Marc, please.\n");
+                    exit(-1);
+                }                
+            } else { 
+                bgScaleBlock = Dim3Int(kPaddedStateCount, kMatrixBlockSize);
+                bgScaleGrid  = Dim3Int(kPatternCount, kCategoryCount/kMatrixBlockSize);
             }
-        } else { 
-            bgScaleBlock = Dim3Int(kPaddedStateCount, kMatrixBlockSize);
-            bgScaleGrid  = Dim3Int(kPatternCount, kCategoryCount/kMatrixBlockSize);
+            if (kCategoryCount % kMatrixBlockSize != 0)
+                bgScaleGrid.y += 1;
+            if (bgScaleGrid.y > 1) { 
+                fprintf(stderr, "Not yet implemented! Try slow reweighing.\n");
+                exit(-1);
+            }        
         }
-        if (kCategoryCount % kMatrixBlockSize != 0)
-            bgScaleGrid.y += 1;
-        if (bgScaleGrid.y > 1) { 
-            fprintf(stderr, "Not yet implemented! Try slow reweighing.\n");
-            exit(-1);
-        }        
+    }    
+
+    // Set up block for site likelihood accumulation
+    if (CPUImplementation) {
+        bgSumSitesBlock = Dim3Int(1);
+    } else {
+        bgSumSitesBlock = Dim3Int(kSumSitesBlockSize);
     }
-    
-    // Set up block/grid for site likelihood accumulation
-    bgSumSitesBlock = Dim3Int(kSumSitesBlockSize);
-    bgSumSitesGrid  = Dim3Int(kPatternCount / kSumSitesBlockSize);
-    if (kPatternCount % kSumSitesBlockSize != 0)
+    bgSumSitesGrid  = Dim3Int(kUnpaddedPatternCount / kSumSitesBlockSize);
+    if (kUnpaddedPatternCount % kSumSitesBlockSize != 0)
         bgSumSitesGrid.x += 1;
-    
+
 }
 
 void KernelLauncher::LoadKernels() {
-	
-	if (kFlags & BEAGLE_FLAG_EIGEN_COMPLEX) {
-		fMatrixMulADB = gpu->GetFunction("kernelMatrixMulADBComplex");
-	} else {
-		fMatrixMulADB = gpu->GetFunction("kernelMatrixMulADB");
-	}
 
+#ifdef FW_OPENCL_TESTING
+	fMatrixMulADB = gpu->GetFunction("kernelMatrixMulADB");
+    fPartialsPartialsByPatternBlockCoherent = gpu->GetFunction(
+            "kernelPartialsPartialsNoScale");
+    fIntegrateLikelihoods = gpu->GetFunction("kernelIntegrateLikelihoods");
+    fSumSites1 = gpu->GetFunction("kernelSumSites1");
+#else
 	//TODO: Epoch Model
 	fMatrixConvolution = gpu ->GetFunction("kernelMatrixConvolution");
 
@@ -152,6 +207,12 @@ void KernelLauncher::LoadKernels() {
     
     fMatrixMulADBSecondDeriv = gpu->GetFunction("kernelMatrixMulADBSecondDeriv");
     
+	if (kFlags & BEAGLE_FLAG_EIGEN_COMPLEX) {
+		fMatrixMulADB = gpu->GetFunction("kernelMatrixMulADBComplex");
+	} else {
+		fMatrixMulADB = gpu->GetFunction("kernelMatrixMulADB");
+	}
+
     fPartialsPartialsByPatternBlockCoherent = gpu->GetFunction(
             "kernelPartialsPartialsNoScale");
     
@@ -266,31 +327,9 @@ void KernelLauncher::LoadKernels() {
     fSumSites1 = gpu->GetFunction("kernelSumSites1");
     fSumSites2 = gpu->GetFunction("kernelSumSites2");
     fSumSites3 = gpu->GetFunction("kernelSumSites3");
+#endif // !FW_OPENCL_TESTING
 }
 
-#ifdef CUDA
-
-void KernelLauncher::PartialsPartialsEdgeLikelihoods(GPUPtr dPartialsTmp,
-                                                     GPUPtr dParentPartials,
-                                                     GPUPtr dChildParials,
-                                                     GPUPtr dTransMatrix,
-                                                     unsigned int patternCount,
-                                                     unsigned int categoryCount) {
-#ifdef BEAGLE_DEBUG_FLOW
-    fprintf(stderr,"\t\tEntering KernelLauncher::PartialsPartialsEdgeLikelihoods\n");
-#endif
-
-    gpu->LaunchKernel(fPartialsPartialsEdgeLikelihoods,
-                               bgPeelingBlock, bgPeelingGrid,
-                               4, 5,
-                               dPartialsTmp, dParentPartials, dChildParials, dTransMatrix,
-                               patternCount);
-
-#ifdef BEAGLE_DEBUG_FLOW
-    fprintf(stderr, "\t\tLeaving  KernelLauncher::PartialsPartialsEdgeLikelihoods\n");
-#endif
-
-}
 
 ///////////////////////////
 //---TODO: Epoch Model---//
@@ -417,45 +456,6 @@ void KernelLauncher::GetTransitionProbabilitiesSquareSecondDeriv(GPUPtr dMatrice
 }
 
 
-#else //OpenCL
-void KernelLauncher::GetTransitionProbabilitiesSquare(GPUPtr dPtr,
-                                                      GPUPtr dEvec,
-                                                      GPUPtr dIevc,
-                                                      GPUPtr dEigenValues,
-                                                      GPUPtr distanceQueue,
-                                                      unsigned int totalMatrix,
-                                                      unsigned int index) {
-#ifdef BEAGLE_DEBUG_FLOW
-    fprintf(stderr, "\t\tEntering KernelLauncher::GetTransitionProbabilitiesSquare\n");
-#endif
-    
-    Dim3Int block(kMultiplyBlockSize, kMultiplyBlockSize); // TODO Construct once
-    Dim3Int grid(kPaddedStateCount / kMultiplyBlockSize,
-            kPaddedStateCount / kMultiplyBlockSize);
-    if (kPaddedStateCount % kMultiplyBlockSize != 0) {
-        grid.x += 1;
-        grid.y += 1;
-    }
-    
-    grid.x *= totalMatrix;
-    
-    // Transposed (interchanged Ievc and Evec)    
-    int parameterCountV = 5;
-    int totalParameterCount = 9;
-    gpu->LaunchKernel(fMatrixMulADB,
-                               block, grid,
-                               parameterCountV, totalParameterCount,
-                               dPtr, dIevc, dEigenValues, dEvec, distanceQueue,
-                               kPaddedStateCount, kPaddedStateCount, totalMatrix,
-                               index);
-    
-#ifdef BEAGLE_DEBUG_FLOW
-    fprintf(stderr, "\t\tLeaving  KernelLauncher::GetTransitionProbabilitiesSquare\n");
-#endif
-}
-#endif
-
-
 void KernelLauncher::PartialsPartialsPruningDynamicCheckScaling(GPUPtr partials1,
                                                            GPUPtr partials2,
                                                            GPUPtr partials3,
@@ -617,7 +617,20 @@ void KernelLauncher::StatesPartialsPruningDynamicScaling(GPUPtr states1,
                                                          int doRescaling) {
 #ifdef BEAGLE_DEBUG_FLOW
     fprintf(stderr, "\t\tEntering KernelLauncher::StatesPartialsPruningDynamicScaling\n");
-#endif    
+#endif
+
+#ifdef FW_OPENCL
+    // fix for Apple CPU OpenCL limitations
+    size_t blockX = bgPeelingBlock.x;
+    size_t gridX  = bgPeelingGrid.x;
+    bool AppleCPUImplementation = false;
+    if (gpu->GetDeviceImplementationCode(-1) == BEAGLE_OPENCL_DEVICE_APPLE_CPU &&
+        kPaddedStateCount == 4) {
+        bgPeelingBlock.x = 1;
+        bgPeelingGrid.x  = gridX * blockX;
+        AppleCPUImplementation = true;
+    }
+#endif
            
     if (doRescaling != 0)    {
         
@@ -664,6 +677,14 @@ void KernelLauncher::StatesPartialsPruningDynamicScaling(GPUPtr states1,
         }
 #endif
     }
+
+#ifdef FW_OPENCL
+    // restore values if used fix for Apple CPU OpenCL limitations
+    if (AppleCPUImplementation) {
+        bgPeelingBlock.x = blockX;
+        bgPeelingGrid.x  = gridX;
+    }
+#endif
     
 #ifdef BEAGLE_DEBUG_FLOW
     fprintf(stderr,"\t\tLeaving  KernelLauncher::StatesPartialsPruningDynamicScaling\n");
@@ -683,7 +704,20 @@ void KernelLauncher::StatesStatesPruningDynamicScaling(GPUPtr states1,
                                                        int doRescaling) {
 #ifdef BEAGLE_DEBUG_FLOW
     fprintf(stderr, "\t\tEntering KernelLauncher::StatesStatesPruningDynamicScaling\n");
-#endif    
+#endif
+
+#ifdef FW_OPENCL
+    // fix for Apple CPU OpenCL limitations
+    size_t blockX = bgPeelingBlock.x;
+    size_t gridX  = bgPeelingGrid.x;
+    bool AppleCPUImplementation = false;
+    if (gpu->GetDeviceImplementationCode(-1) == BEAGLE_OPENCL_DEVICE_APPLE_CPU &&
+        kPaddedStateCount == 4) {
+        bgPeelingBlock.x = 1;
+        bgPeelingGrid.x  = gridX * blockX;
+        AppleCPUImplementation = true;
+    }
+#endif
        
     if (doRescaling != 0)    {
         
@@ -731,6 +765,14 @@ void KernelLauncher::StatesStatesPruningDynamicScaling(GPUPtr states1,
         }
 #endif
     }
+
+#ifdef FW_OPENCL
+    // restore values if used fix for Apple CPU OpenCL limitations
+    if (AppleCPUImplementation) {
+        bgPeelingBlock.x = blockX;
+        bgPeelingGrid.x  = gridX;
+    }
+#endif
     
 #ifdef BEAGLE_DEBUG_FLOW
     fprintf(stderr, "\t\tLeaving  KernelLauncher::StatesStatesPruningDynamicScaling\n");
@@ -785,6 +827,29 @@ void KernelLauncher::IntegrateLikelihoodsDynamicScalingSecondDeriv(GPUPtr dResul
 #endif
 }
 
+
+void KernelLauncher::PartialsPartialsEdgeLikelihoods(GPUPtr dPartialsTmp,
+                                                     GPUPtr dParentPartials,
+                                                     GPUPtr dChildParials,
+                                                     GPUPtr dTransMatrix,
+                                                     unsigned int patternCount,
+                                                     unsigned int categoryCount) {
+#ifdef BEAGLE_DEBUG_FLOW
+    fprintf(stderr,"\t\tEntering KernelLauncher::PartialsPartialsEdgeLikelihoods\n");
+#endif
+
+    gpu->LaunchKernel(fPartialsPartialsEdgeLikelihoods,
+                               bgPeelingBlock, bgPeelingGrid,
+                               4, 5,
+                               dPartialsTmp, dParentPartials, dChildParials, dTransMatrix,
+                               patternCount);
+
+#ifdef BEAGLE_DEBUG_FLOW
+    fprintf(stderr, "\t\tLeaving  KernelLauncher::PartialsPartialsEdgeLikelihoods\n");
+#endif
+
+}
+
 void KernelLauncher::PartialsPartialsEdgeLikelihoodsSecondDeriv(GPUPtr dPartialsTmp,
                                                                 GPUPtr dFirstDerivTmp,
                                                                 GPUPtr dSecondDerivTmp,
@@ -823,11 +888,32 @@ void KernelLauncher::StatesPartialsEdgeLikelihoods(GPUPtr dPartialsTmp,
     fprintf(stderr,"\t\tEntering KernelLauncher::StatesPartialsEdgeLikelihoods\n");
 #endif
     
+#ifdef FW_OPENCL
+    // fix for Apple CPU OpenCL limitations
+    size_t blockX = bgPeelingBlock.x;
+    size_t gridX  = bgPeelingGrid.x;
+    bool AppleCPUImplementation = false;
+    if (gpu->GetDeviceImplementationCode(-1) == BEAGLE_OPENCL_DEVICE_APPLE_CPU &&
+        kPaddedStateCount == 4) {
+        bgPeelingBlock.x = 1;
+        bgPeelingGrid.x  = gridX * blockX;
+        AppleCPUImplementation = true;
+    }
+#endif
+
     gpu->LaunchKernel(fStatesPartialsEdgeLikelihoods,
                                bgPeelingBlock, bgPeelingGrid,
                                4, 5,
                                dPartialsTmp, dParentPartials, dChildStates, dTransMatrix,
                                patternCount);  
+
+#ifdef FW_OPENCL
+    // restore values if used fix for Apple CPU OpenCL limitations
+    if (AppleCPUImplementation) {
+        bgPeelingBlock.x = blockX;
+        bgPeelingGrid.x  = gridX;
+    }
+#endif
     
 #ifdef BEAGLE_DEBUG_FLOW
     fprintf(stderr, "\t\tLeaving  KernelLauncher::StatesPartialsEdgeLikelihoods\n");
@@ -848,6 +934,19 @@ void KernelLauncher::StatesPartialsEdgeLikelihoodsSecondDeriv(GPUPtr dPartialsTm
 #ifdef BEAGLE_DEBUG_FLOW
     fprintf(stderr,"\t\tEntering KernelLauncher::StatesPartialsEdgeLikelihoodsSecondDeriv\n");
 #endif
+
+#ifdef FW_OPENCL
+    // fix for Apple CPU OpenCL limitations
+    size_t blockX = bgPeelingBlock.x;
+    size_t gridX  = bgPeelingGrid.x;
+    bool AppleCPUImplementation = false;
+    if (gpu->GetDeviceImplementationCode(-1) == BEAGLE_OPENCL_DEVICE_APPLE_CPU &&
+        kPaddedStateCount == 4) {
+        bgPeelingBlock.x = 1;
+        bgPeelingGrid.x  = gridX * blockX;
+        AppleCPUImplementation = true;
+    }
+#endif
     
     gpu->LaunchKernel(fStatesPartialsEdgeLikelihoodsSecondDeriv,
                                bgPeelingBlock, bgPeelingGrid,
@@ -861,6 +960,14 @@ void KernelLauncher::StatesPartialsEdgeLikelihoodsSecondDeriv(GPUPtr dPartialsTm
     fprintf(stderr, "\t\tLeaving  KernelLauncher::StatesPartialsEdgeLikelihoodsSecondDeriv\n");
 #endif
     
+#ifdef FW_OPENCL
+    // restore values if used fix for Apple CPU OpenCL limitations
+    if (AppleCPUImplementation) {
+        bgPeelingBlock.x = blockX;
+        bgPeelingGrid.x  = gridX;
+    }
+#endif
+
 }
 
 void KernelLauncher::AccumulateFactorsDynamicScaling(GPUPtr dScalingFactors,
@@ -1115,7 +1222,7 @@ void KernelLauncher::SumSites1(GPUPtr dArray1,
 #ifdef BEAGLE_DEBUG_FLOW
     fprintf(stderr, "\t\tEntering KernelLauncher::SumSites1\n");
 #endif
-    
+
     int parameterCountV = 3;
     int totalParameterCount = 4;
     gpu->Synchronize();  
@@ -1140,6 +1247,7 @@ void KernelLauncher::SumSites2(GPUPtr dArray1,
 #ifdef BEAGLE_DEBUG_FLOW
     fprintf(stderr, "\t\tEntering KernelLauncher::SumSites2\n");
 #endif
+
     int parameterCountV = 5;
     int totalParameterCount = 6;
     gpu->Synchronize();
@@ -1182,6 +1290,7 @@ void KernelLauncher::SumSites3(GPUPtr dArray1,
     
 }
 
+}; // namespace
 
 
 
